import remoteapi

"""
TestDataTable.py

Unit test for Remote API that creates a single client and executes
several scenario-based PIData commands remotely.  It creates a
data table, get the data table and then remove the data table
"""

client = remoteapi.DiscoveryClient().createClient()

if client is None:
	raise Exception("Failed to connect.")
pi = client.pluginApi()
piData = pi.data()
pi.config().setResponsesEnabled(True)

f = piData.isScenarioInitialized()
if not f.isOk(1000):
	raise Exception("Failed isScenarioInitialized")

if f.value():
	f = piData.deleteScenario()
	if not f.isOk(5000):
		raise Exception("Failed to delete scenario")

f = piData.initializeScenario("Python Test", 0, 0, 0)
if not f.isOk(5000):
	raise Exception("Failed to initialize scenario")

f = piData.setOriginLLA(0.1, 0.2, 1000)
if not f.isOk(1000):
	raise Exception("Failed to set OriginLLA")

f = piData.getOriginLLA()
if not f.isOk(1000):
	raise Exception("Failed to receive Origin LLA")
if f.value()[0] != 0.1:
	raise Exception("Failed to receive expected latitude")
if f.value()[1] != 0.2:
	raise Exception("Failed to receive expected longitude")
if f.value()[2] != 1000:
	raise Exception("Failed to receive expected altitude")

f = piData.setReferenceYear(2013)
if not f.isOk(1000):
	raise Exception("Failed to set ref year")
f = piData.getReferenceYear()
if not f.isOk(1000):
	raise Exception("Failed to get ref year")
if f.value() != 2013:
	raise Exception("Failed to receive expected reference year")

# Create a scenario data table
f = piData.createDataTable(0, "New Table")
if not f.isOk(1000):
	raise Exception("Failed to create data table")
tableId = f.value()

# Add some columns
f = piData.addColumn(tableId, "Column 1", 10, 20) # Double, meters
if not f.isOk(1000):
	raise Exception("Failed to add column to data table")

f = piData.addColumn(tableId, "Column 2", 10, 20) # Double, meters
if not f.isOk(1000):
	raise Exception("Failed to add column to data table")

f = piData.addColumn(tableId, "Column 3", 10, 20) # Double, meters
if not f.isOk(1000):
	raise Exception("Failed to add column to data table")

# Add some rows
f = piData.addRow(tableId, 0.0, {1.0, 2.0, 3.0})
if not f.isOk(1000):
	raise Exception("Failed to add row to data table")

f = piData.addRow(tableId, 10.0, {10.0, 20.0, 30.0})
if not f.isOk(1000):
	raise Exception("Failed to add row to data table")

f = piData.addRow(tableId, 100.0, {100.0, 200.0, 300.0})
if not f.isOk(1000):
	raise Exception("Failed to add row to data table")

f = piData.addRow(tableId, 1000.0, {1000.0, 2000.0, 3000.0})
if not f.isOk(1000):
	raise Exception("Failed to add row to data table")

# Get columns
f = piData.getColumns(tableId)
if not f.isOk(1000):
	raise Exception("Failed to get columns")
if len(f.value()) != 3:
	raise Exception("Failed to get correct number of columns")

# Get rows
f = piData.getRows(tableId, 0.0, 1000.0)
if not f.isOk(1000):
	raise Exception("Failed to get rows")
if len(f.value()) != 4:
	raise Exception("Failed to get correct number of rows")

# Get table name
f = piData.getTableName(tableId)
if not f.isOk(1000):
	raise Exception("Failed to get table name")
if f.value() != "New Table":
	raise Exception("Failed to get correct table name")

f = piData.getTableList(0)
if not f.isOk(1000):
	raise Exception("Failed to get table list")
if len(f.value().uniqueId) != 1:
	raise Exception("Failed to get correct number of tables")

f = piData.removeDataTable(tableId)
if not f.isOk(1000):
	raise Exception("Failed to remove data table")

client.close()
print("Test successful")
