import remoteapi

"""
TestLLA.py

Unit test for Remote API that creates a single client and executes
several LLA-based SIMDIS commands remotely.
"""
client = remoteapi.DiscoveryClient().createClient()
try:

	if client is None:
		raise Exception("Failed to connect.")
	pi = client.pluginApi()

	f = pi.data().setOriginLLA(0,0,0)
	if not f.isOk(1000):
		raise Exception("Failed to setOriginLLA: " + f.responseDescription())

	f = pi.simdis().geodeticToScreenXY(0,0,0)
	if not f.isOk(1000):
		raise Exception("Failed geodeticToScreenXY: " + f.responseDescription())
	if f.value().sx < 0.0 or f.value().sy < 0.0:
		raise Exception("geodeticToScreenXY returned negative values for screen coordinates")

	# Get altitude for a spot in Hawaii with elevation information
	f = pi.simdis().getAltitudeForLL(0.386202, -2.785, 1)
	if not f.isOk(1000):
		raise Exception("Failed getAltitudeForLL: " + f.responseDescription())
	if f.value() == 0.0:
		raise Exception("getAltitudeForLL returned 0.0. Is elevation data loaded for Hawaii?")

	print("Test successful")
finally:
	client.close()

