import RemoteApi_pb2
import PIDataScenario_pb2
import remoteapi
import sys

"""
TestLoadDISCNFile.py

Unit test for Remote API that loads a single DISCN file.
"""
def WaitForOK(f, waitAmount):
	secondsRemaining = waitAmount
	# Only wait 1 second at a time, printing a '.' each second
	while secondsRemaining > 0 and not f.waitForResponse(1000):
		secondsRemaining -= 1
		sys.stdout.write('.')
		sys.stdout.flush()
		# Break out if we got a negative response
		if f.responseReceived():
			break

	if not f.isOk(0):
		raise Exception("Command Failed")

	if secondsRemaining != waitAmount:
		print("")

client = remoteapi.DiscoveryClient().createClient()
if client is None:
	raise Exception("Failed to connect.")

pi = client.pluginApi()
piData = pi.data()
piSimdis = pi.simdis()
waitAmount = 45

# Set up scenario
f = piData.isScenarioInitialized()
if not f.isOk(1000):
    raise Exception("Failed isScenarioInitialized")

if f.value():
    f = piData.deleteScenario()
    if not f.isOk(1000):
        raise Exception("Failed to delete scenario")

f = piData.initializeScenario("Test Load DISCN File", 0, 0, 0)
if not f.isOk(1000):
    raise Exception("Failed to initialize scenario")

# Set clock to live mode
f = piData.clockSetMode(PIDataScenario_pb2.CLOCKMODE_FREEWHEEL, 0.0)
if not f.isOk(1000):
    raise Exception("Failed to setClockMode: " + f.responseDescription())

# Enforce live mode before loading file
f = piData.clockGetMode()
if not f.isOk(1000):
    raise Exception("Failed clockGetMode")
if not f.value() in [PIDataScenario_pb2.CLOCKMODE_FREEWHEEL, PIDataScenario_pb2.CLOCKMODE_SIMULATION]:
    raise Exception("SIMDIS must be in live mode to test loading DISCN files")

# Loading an .asi file with a GOG as DISCN to test
f = piSimdis.loadDISCNFile("$(SIMDIS_DIR)/demos/SIMDIS/import/testECEF.asi")
print("Requested load of data file, waiting up to " + str(waitAmount) + " seconds")
WaitForOK(f, waitAmount)

# Ensure the scenario got loaded properly with a find-platform
f = piData.findPlatform('Rat Patrol')
if not f.isOk(1000) or f.value() == 0:
	raise Exception("Failed findPlatform")

client.close()
print("Test successful")
