import math
import RemoteApi_pb2
import remoteapi

"""
TestPreference.py

Unit test for Remote API setPreference and getPreference functions
"""

client = remoteapi.DiscoveryClient().createClient()
if client is None:
	raise Exception("Failed to connect.")

pi = client.pluginApi()
piData = pi.data()
piSimdis = pi.simdis()

f = piData.deleteScenario()
if not f.isOk(1000):
	raise Exception("Failed deleteScenario")

# Load data file is unique because it can take a while to respond...
f = piData.loadDataFile("$(SIMDIS_DIR)/demos/SIMDIS/import/wake.asi")
print("Requested load of data file, waiting up to 50 seconds")
if not f.isOk(50000):
	raise Exception("Failed loadDataFile: " + f.responseDescription())

# Find both the beam and a platform
f = piData.findPlatform('PC-12')
if not f.isOk(1000) or f.value() == 0:
	raise Exception("Failed findPlatform on PC-12")
pc12Id = f.value()
f = piData.findBeam("Torp Beam")
if not f.isOk(1000) or f.value() == 0:
	raise Exception("Failed findBeam on Torp Beam")
torpBeamId = f.value()

# Set the icon on PC-12 to A6V with purple override color
f = piSimdis.setPreference(pc12Id, "icon", "A6V.png")
if not f.isOk(4000): # wait a few seconds for disk operation if necessary
	raise Exception("Failed setPreference(icon) on PC-12")
f = piSimdis.setPreference(pc12Id, "commonPrefs.overrideColor", "0xff00ffff") # Purple
if not f.isOk(1000):
	raise Exception("Failed setPreference(commonPrefs.overrideColor) on PC-12")
f = piSimdis.setPreference(pc12Id, "commonPrefs.useOverrideColor", "1")
if not f.isOk(1000):
	raise Exception("Failed setPreference(commonPrefs.useOverrideColor) on PC-12")

# Turn on beam animation for the torp beam
f = piSimdis.setPreference(torpBeamId, "animate", "True")
if not f.isOk(1000):
	raise Exception("Failed setPreference(animate) on Torp Beam")
f = piSimdis.setPreference(torpBeamId, "pulseLength", "1500")
if not f.isOk(1000):
	raise Exception("Failed setPreference(pulseLength) on Torp Beam")
f = piSimdis.setPreference(torpBeamId, "pulseStipple", "0xc000")
if not f.isOk(1000):
	raise Exception("Failed setPreference(pulseStipple) on Torp Beam")

# Test getPreference
f = piSimdis.getPreference(torpBeamId, "pulseLength")
if not f.isOk(1000):
	raise Exception("Failed getPreference(pulseLength) on Torp Beam")
# Note that the value will have string formatting
if not float(f.value()) == 1500.0:
	raise Exception("Expected pulse length of 1500.0, got instead: " + f.value())

# Test savePreferences and saveEntityPreferences
# Start with a known state on Torp Beam
f = piSimdis.setPreference(torpBeamId, "pulseLength", "1500")
if not f.isOk(1000):
	raise Exception("Failed setPreference(pulseLength) on Torp Beam")
f = piSimdis.savePreferencesStream()
if not f.isOk(1000):
	raise Exception("Failed savePreferences")
allPrefs_Length1500 = f.value()

# Update the value
f = piSimdis.setPreference(torpBeamId, "pulseLength", "2000")
if not f.isOk(1000):
	raise Exception("Failed setPreference(pulseLength) on Torp Beam")
# Save the single prefs for the beam
f = piSimdis.saveEntityPreferencesStream(torpBeamId)
if not f.isOk(1000):
	raise Exception("Failed saveEntityPreferences")
singlePrefs_Length2000 = f.value()

# Test the prefs state to establish a starting point
f = piSimdis.getPreference(torpBeamId, "pulseLength")
if not f.isOk(1000) or not float(f.value()) == 2000.0:
	raise Exception("Expected pulse length of 2000.0, got instead: " + f.value())

# Test loadPreferencesStream and loadEntityPreferencesStream
f = piSimdis.loadPreferencesStream(allPrefs_Length1500)
if not f.isOk(1000):
	raise Exception("Failed loadPreferences()")
f = piSimdis.getPreference(torpBeamId, "pulseLength")
if not f.isOk(1000) or not float(f.value()) == 1500.0:
	raise Exception("Expected pulse length of 1500.0, got instead: " + f.value())
f = piSimdis.loadEntityPreferencesStream(torpBeamId, singlePrefs_Length2000)
if not f.isOk(1000):
	raise Exception("Failed loadPreferences()")
f = piSimdis.getPreference(torpBeamId, "pulseLength")
if not f.isOk(1000) or not float(f.value()) == 2000.0:
	raise Exception("Expected pulse length of 2000.0, got instead: " + f.value())

f = piSimdis.setPreference(pc12Id, "commonPrefs.labelPrefs.useValues", "0")
if not f.isOk(1000):
	raise Exception("Failed setPreference(commonPrefs.labelPrefs.useValues) on PC-12")

f = piSimdis.setPreference(pc12Id, "animateDofNodes", "False")
if not f.isOk(1000):
	raise Exception("Failed setPreference(animateDofNodes) on PC-12")

f = piSimdis.setPreference(pc12Id, "polygonMode", "0x1B02")
if not f.isOk(1000):
	raise Exception("Failed setPreference(polygonMode) on PC-12")

# Load data file is unique because it can take a while to respond...
f = piData.loadDataFile("$(SIMDIS_DIR)/demos/SIMDIS/Examples/projectedTexture/projectedTexture.asi")
print("Requested load of data file, waiting up to 50 seconds")
if not f.isOk(50000):
	raise Exception("Failed loadDataFile: " + f.responseDescription())

f = piData.findEntity("UAV Camera Feed 01")
if not f.isOk(1000) or f.value() == 0:
	raise Exception("Failed findEntity on UAV Camera Feed 01")
projectorId = f.value()

f = piSimdis.setPreference(projectorId, "overrideFov", "True")
if not f.isOk(1000):
	raise Exception("Failed setPreference(overrideFov) on UAV Camera Feed 01")
	
f = piSimdis.setPreference(projectorId, "overrideFovAngle", "0.2")
if not f.isOk(1000):
	raise Exception("Failed setPreference(overrideFovAngle) on UAV Camera Feed 01")
	
# All done, successful if we got here
client.close()
print("Test successful")
