import remoteapi

"""
TestRemoteLogging.py

Unit test for Remote API that creates a single client and configures logging
before executing several scenario-based commands to have data to put into
the logging files.  It creates a scenario, initializes values, checks
values, then continues to execute other scenario commands.
End result will create two command log files in the SIMDIS_HOME directory
"""

# Convenience variables to store wait times
waitTimeShort = 1000
waitTimeLong = 10000

logLocation = "$(SIMDIS_HOME)\\testLog.raf"

try:
	client = remoteapi.DiscoveryClient().createClient()

	if client is None:
		raise Exception("Failed to connect.")

	pi = client.pluginApi()
	piConfig = pi.config()
	piData = pi.data()

	# Turn on command logging. A timestamped file should be in the SIMDIS_HOME dir
	piConfig.logToFile(logLocation)

	# Arbitrary commands to fill log file ---------
	f = piData.isScenarioInitialized()
	if not f.isOk(waitTimeShort):
		raise Exception("Failed isScenarioInitialized")

	if f.value():
		f = piData.deleteScenario()
		if not f.isOk(waitTimeLong):
			raise Exception("Failed to delete scenario")

	f = piData.initializeScenario("Python Test", 0, 0, 0)
	if not f.isOk(waitTimeLong):
		raise Exception("Failed to initialize scenario")

	f = piData.setOriginLLA(0.1, 0.2, 1000)
	if not f.isOk(waitTimeShort):
		raise Exception("Failed to set OriginLLA")

	f = piData.getOriginLLA()
	if not f.isOk(waitTimeShort):
		raise Exception("Failed to receive Origin LLA")
	if f.value()[0] != 0.1:
		raise Exception("Failed to receive expected latitude")
	if f.value()[1] != 0.2:
		raise Exception("Failed to receive expected longitude")
	if f.value()[2] != 1000:
		raise Exception("Failed to receive expected altitude")

	#----------------------------------------------
	# Re-activate logging to cause the creation of a second file.
	# Lack of a second file is a test failure
	piConfig.logToFile(logLocation)

	# Arbitrary commands to fill log file ---------
	f = piData.isScenarioInitialized()
	if not f.isOk(waitTimeShort):
		raise Exception("Failed isScenarioInitialized")

	if f.value():
		f = piData.deleteScenario()
		if not f.isOk(waitTimeLong):
			raise Exception("Failed to delete scenario")

	f = piData.initializeScenario("Python Test", 0, 0, 0)
	if not f.isOk(waitTimeLong):
		raise Exception("Failed to initialize scenario")

	f = piData.setOriginLLA(0.1, 0.2, 1000)
	if not f.isOk(waitTimeShort):
		raise Exception("Failed to set OriginLLA")

	f = piData.getOriginLLA()
	if not f.isOk(waitTimeShort):
		raise Exception("Failed to receive Origin LLA")
	if f.value()[0] != 0.1:
		raise Exception("Failed to receive expected latitude")
	if f.value()[1] != 0.2:
		raise Exception("Failed to receive expected longitude")
	if f.value()[2] != 1000:
		raise Exception("Failed to receive expected altitude")

	# Turn off logging to close the file
	piConfig.stopLogging()

	client.close()

# Attempt to close the client before ending if we hit an exception
finally:
	client.close()
