import RemoteApi_pb2
import Constants_pb2
import PICommon_pb2
try:
  from . import PluginApi
except ImportError:
  import PluginApi

ReturnCodeOnly = lambda r: r.response.returnCode

class PICommonSender(object):
	def __init__(self, client):
		self.client_ = client

	def executeAction(self, action):
		msg = RemoteApi_pb2.Command()
		msg.piCommon.executeAction = action
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def actionState(self, name):
		msg = RemoteApi_pb2.Command()
		msg.piCommon.actionState = name
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piCommon.actionState)

	def setHotKey(self, actionName, hotKey, hotKeyNum, force):
		msg = RemoteApi_pb2.Command()
		msg.piCommon.setHotKey.actionName = actionName
		msg.piCommon.setHotKey.hotKey = hotKey
		msg.piCommon.setHotKey.hotKeyNum = hotKeyNum
		msg.piCommon.setHotKey.force = force
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getHotKey(self, actionName, hotKeyNum):
		msg = RemoteApi_pb2.Command()
		msg.piCommon.getHotKey.actionName = actionName
		msg.piCommon.getHotKey.hotKeyNum = hotKeyNum
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piCommon.id)

	def loadHotKeyFile(self, filename):
		msg = RemoteApi_pb2.Command()
		msg.piCommon.loadHotKeyFile = filename
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def saveHotKeyFile(self, filename):
		msg = RemoteApi_pb2.Command()
		msg.piCommon.saveHotKeyFile = filename
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def rangeCalculateValue(self, beginId, endId, calcType, earthModel=Constants_pb2.EARTHMODEL_WGS84):
		msg = RemoteApi_pb2.Command()
		msg.piCommon.rangeCalculateValue.beginId = beginId
		msg.piCommon.rangeCalculateValue.endId = endId
		msg.piCommon.rangeCalculateValue.calcType = calcType
		msg.piCommon.rangeCalculateValue.earthModel = earthModel
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piCommon.rangeCalculation)

	def setNotifyLevel(self, severity):
		msg = RemoteApi_pb2.Command()
		msg.piCommon.setNotifyLevel = severity
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getNotifyLevel(self):
		msg = RemoteApi_pb2.Command()
		msg.piCommon.getNotifyLevel = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piCommon.notifySeverity)

	def notify(self, text, severity=PICommon_pb2.NOTIFY_NOTICE, overrideChannel=None):
		msg = RemoteApi_pb2.Command()
		msg.piCommon.notify.text = text
		msg.piCommon.notify.severity = severity
		if overrideChannel is not None:
			msg.piCommon.notify.overrideChannel = overrideChannel
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def createOverlayText(self, text, x, y):
		msg = RemoteApi_pb2.Command()
		msg.piCommon.createOverlayText.text = text
		msg.piCommon.createOverlayText.x = x
		msg.piCommon.createOverlayText.y = y
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piCommon.id)

	def updateOverlayText(self, id, text, x, y):
		msg = RemoteApi_pb2.Command()
		msg.piCommon.updateOverlayText.id = id
		msg.piCommon.updateOverlayText.text.text = text
		msg.piCommon.updateOverlayText.text.x = x
		msg.piCommon.updateOverlayText.text.y = y
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getOverlayText(self, id):
		msg = RemoteApi_pb2.Command()
		msg.piCommon.getOverlayText = id
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piCommon.overlayText)

	def removeOverlayText(self, id):
		msg = RemoteApi_pb2.Command()
		msg.piCommon.removeOverlayText = id
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def setViewport(self, viewport):
		msg = RemoteApi_pb2.Command()
		msg.piCommon.setViewport = viewport
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getViewport(self, viewportNum):
		msg = RemoteApi_pb2.Command()
		msg.piCommon.getViewport = viewportNum
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piCommon.stringValue)

	def getNumViewports(self):
		msg = RemoteApi_pb2.Command()
		msg.piCommon.getNumViewports = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piCommon.intValue)

	def nextViewport(self):
		msg = RemoteApi_pb2.Command()
		msg.piCommon.nextViewport = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def deleteViewport(self, viewport):
		msg = RemoteApi_pb2.Command()
		msg.piCommon.deleteViewport = viewport
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def deleteAllViewports(self):
		msg = RemoteApi_pb2.Command()
		msg.piCommon.deleteAllViewports = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def toggleViewport(self, viewport, showIt):
		msg = RemoteApi_pb2.Command()
		msg.piCommon.toggleViewport.viewport = viewport
		msg.piCommon.toggleViewport.showIt = showIt
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def createViewport(self, percentWay, viewport, leftX, width, bottomY, height):
		msg = RemoteApi_pb2.Command()
		msg.piCommon.createViewport.percentWay = percentWay
		msg.piCommon.createViewport.viewport = viewport
		msg.piCommon.createViewport.leftX = leftX
		msg.piCommon.createViewport.width = width
		msg.piCommon.createViewport.bottomY = bottomY
		msg.piCommon.createViewport.height = height
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def checkEarthEclipse(self, id):
		msg = RemoteApi_pb2.Command()
		msg.piCommon.checkEarthEclipse = id
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piCommon.intValue)

	def loadPrefsFile(self, filename):
		msg = RemoteApi_pb2.Command()
		msg.piCommon.loadPrefsFile = filename
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def savePrefsFile(self, filename):
		msg = RemoteApi_pb2.Command()
		msg.piCommon.savePrefsFile = filename
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def showToastNotification(self, messageText, duration):
		msg = RemoteApi_pb2.Command()
		msg.piCommon.showToastNotification.toastText = messageText
		msg.piCommon.showToastNotification.duration = duration
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def startRecording(self, filename, forceOverwrite, interval, length, organizeByDate=None, autoStartTime=None, autoHaltTime=None):
		msg = RemoteApi_pb2.Command()
		msg.piCommon.startRecording.filename = filename
		msg.piCommon.startRecording.forceOverwrite = forceOverwrite
		msg.piCommon.startRecording.interval = interval
		msg.piCommon.startRecording.length = length
		if (organizeByDate != None):
			msg.piCommon.startRecording.organizeByDate = organizeByDate
		if (autoStartTime!= None):
			msg.piCommon.startRecording.autoStartTime = autoStartTime
		if (autoHaltTime != None):
			msg.piCommon.startRecording.autoHaltTime = autoHaltTime
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piCommon.recordingState)

	def stopRecording(self):
		msg = RemoteApi_pb2.Command()
		msg.piCommon.stopRecording = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piCommon.recordingState)

	def setSettingValueStr(self, name, value):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.setSettingsValues.names.append(name)
		msg.piSimdis.setSettingsValues.settingsOneOfs.add().strValue = value
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def setSettingValueInt(self, name, value):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.setSettingsValues.names.append(name)
		msg.piSimdis.setSettingsValues.settingsOneOfs.add().intValue = value
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def setSettingValueDouble(self, name, value):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.setSettingsValues.names.append(name)
		msg.piSimdis.setSettingsValues.settingsOneOfs.add().doubleValue = value
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def setSettingValueBool(self, name, value):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.setSettingsValues.names.append(name)
		msg.piSimdis.setSettingsValues.settingsOneOfs.add().boolValue = value
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def setSettingValueColor(self, name, value):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.setSettingsValues.names.append(name)
		msg.piSimdis.setSettingsValues.settingsOneOfs.add().rgbaValue = value
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def setSettingValueStringVec(self, name, value):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.setSettingsValues.names.append(name)
		outVec = msg.piSimdis.setSettingsValues.settingsOneOfs.add().strVecValue.value
		for str in value:
			outVec.append(str)
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getSettingValueStr(self, name, defaultValue = "default"):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.getSettingsValues.names.append(name)
		msg.piSimdis.getSettingsValues.settingsOneOfs.add().strValue = defaultValue
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.settingsValues.settingsOneOfs[0].strValue)

	def getSettingValueInt(self, name, defaultValue = 0):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.getSettingsValues.names.append(name)
		msg.piSimdis.getSettingsValues.settingsOneOfs.add().intValue = defaultValue
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.settingsValues.settingsOneOfs[0].intValue)

	def getSettingValueDouble(self, name, defaultValue = 0.0):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.getSettingsValues.names.append(name)
		msg.piSimdis.getSettingsValues.settingsOneOfs.add().doubleValue = defaultValue
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.settingsValues.settingsOneOfs[0].doubleValue)

	def getSettingValueBool(self, name, defaultValue = False):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.getSettingsValues.names.append(name)
		msg.piSimdis.getSettingsValues.settingsOneOfs.add().boolValue = defaultValue
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.settingsValues.settingsOneOfs[0].boolValue)

	def getSettingValueColor(self, name, defaultValue = 0x00000000):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.getSettingsValues.names.append(name)
		msg.piSimdis.getSettingsValues.settingsOneOfs.add().rgbaValue = defaultValue
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.settingsValues.settingsOneOfs[0].rgbaValue)

	def getSettingValueStringVec(self, name, defaultValue = [""]):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.getSettingsValues.names.append(name)
		oneOf = msg.piSimdis.getSettingsValues.settingsOneOfs.add()
		for str in defaultValue:
			oneOf.strVecValue.value.append(str)
		# In Python Protobuf we can't create the one-of entry without a value
		if len(defaultValue) == 0:
			oneOf.strVecValue.value.append("")
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.settingsValues.settingsOneOfs[0].strVecValue.value)

	def doesSettingExist(self, name):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.doesSettingExist = name
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.state)
