import RemoteApi_pb2
try:
  from . import PluginApi
except ImportError:
  import PluginApi

ReturnCodeOnly = lambda r: r.response.returnCode

class PISIMDISSender(object):
	def __init__(self, client):
		self.client_ = client

	# Deprecated, use PIDataSender.py loadDataFile()
	def loadDataFile(self, filename):
		return self.client_.pluginApi().data().loadDataFile(filename)

	def saveDataFile(self, filename, options):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.saveDataFile.filename = filename
		for tag, value in options.items():
			option = msg.piSimdis.saveDataFile.options.add()
			option.tag = str(tag)
			option.value = str(value)
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getPOD(self, id, azimRad, gndRngMeters, hgtMeters):
		msg = RemoteApi_pb2.Command()
		getPOD = msg.piSimdis.rfprop.getPOD
		getPOD.beamId = id
		getPOD.azimRad = azimRad
		getPOD.gndRngMeters = gndRngMeters
		getPOD.hgtMeters = hgtMeters
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.rfprop.calculatedValue)

	def getLoss(self, id, azimRad, gndRngMeters, hgtMeters):
		msg = RemoteApi_pb2.Command()
		getLoss = msg.piSimdis.rfprop.getLoss
		getLoss.beamId = id
		getLoss.azimRad = azimRad
		getLoss.gndRngMeters = gndRngMeters
		getLoss.hgtMeters = hgtMeters
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.rfprop.calculatedValue)

	def getPPF(self, id, azimRad, gndRngMeters, hgtMeters):
		msg = RemoteApi_pb2.Command()
		getPPF = msg.piSimdis.rfprop.getPPF
		getPPF.beamId = id
		getPPF.azimRad = azimRad
		getPPF.gndRngMeters = gndRngMeters
		getPPF.hgtMeters = hgtMeters
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.rfprop.calculatedValue)

	def getSNR(self, id, azimRad, slantRngMeters, hgtMeters, xmtGaindB, recGaindB, rcsSqm, gndRngMeters):
		msg = RemoteApi_pb2.Command()
		getSNR = msg.piSimdis.rfprop.getSNR
		getSNR.beamId = id
		getSNR.azimRad = azimRad
		getSNR.slantRngMeters = slantRngMeters
		getSNR.hgtMeters = hgtMeters
		getSNR.xmtGaindB = xmtGaindB
		getSNR.recGaindB = recGaindB
		getSNR.rcsSqm = rcsSqm
		getSNR.gndRngMeters = gndRngMeters
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.rfprop.calculatedValue)

	def getCNR(self, id, azimRad, gndRngMeters):
		msg = RemoteApi_pb2.Command()
		getCNR = msg.piSimdis.rfprop.getCNR
		getCNR.beamId = id
		getCNR.azimRad = azimRad
		getCNR.gndRngMeters = gndRngMeters
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.rfprop.calculatedValue)

	def getOneWayPower(self, id, azimRad, slantRngMeters, hgtMeters, xmtGaindB, gndRngMeters, rcvGaindB):
		msg = RemoteApi_pb2.Command()
		getOneWayPower = msg.piSimdis.rfprop.getOneWayPower
		getOneWayPower.beamId = id
		getOneWayPower.azimRad = azimRad
		getOneWayPower.slantRngMeters = slantRngMeters
		getOneWayPower.hgtMeters = hgtMeters
		getOneWayPower.xmtGaindB = xmtGaindB
		getOneWayPower.gndRngMeters = gndRngMeters
		getOneWayPower.rcvGaindB = rcvGaindB
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.rfprop.calculatedValue)

	def getReceivedPower(self, id, azimRad, slantRngMeters, hgtMeters, xmtGaindB, recGaindB, rcsSqm, gndRngMeters):
		msg = RemoteApi_pb2.Command()
		getReceivedPower = msg.piSimdis.rfprop.getReceivedPower
		getReceivedPower.beamId = id
		getReceivedPower.azimRad = azimRad
		getReceivedPower.slantRngMeters = slantRngMeters
		getReceivedPower.hgtMeters = hgtMeters
		getReceivedPower.xmtGaindB = xmtGaindB
		getReceivedPower.recGaindB = recGaindB
		getReceivedPower.rcsSqm = rcsSqm
		getReceivedPower.gndRngMeters = gndRngMeters
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.rfprop.calculatedValue)

	def loadRangeConfiguration(self, configuration, append, filename):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.rangeCalculations.loadConfiguration.configuration = configuration
		msg.piSimdis.rangeCalculations.loadConfiguration.append = append
		msg.piSimdis.rangeCalculations.loadConfiguration.filename = filename
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getRangeConfiguration(self):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.rangeCalculations.getConfiguration = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.rangeCalculations.configuration)

	def removeAll(self):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.rangeCalculations.removeAll = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def calculateEntityValue(self, type, id, time):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.rangeCalculations.calculate.type = type
		msg.piSimdis.rangeCalculations.calculate.id = id
		msg.piSimdis.rangeCalculations.calculate.time = time
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.rangeCalculations.values)

	def setPreference(self, id, fieldPath, value):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.setPreferences.id = id
		pref = msg.piSimdis.setPreferences.preferences.add()
		pref.fieldPath = fieldPath
		pref.value = value
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getPreference(self, id, fieldPath):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.getPreferences.id = id
		msg.piSimdis.getPreferences.fieldPaths.append(fieldPath)
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.preferenceValues[0])

	def getRCSdB(self, id, freqMhz, relAzimuthRad, relElevationRad, polarity):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.getRCSdB.id = id
		msg.piSimdis.getRCSdB.freqMhz = freqMhz
		msg.piSimdis.getRCSdB.relAzimuthRad = relAzimuthRad
		msg.piSimdis.getRCSdB.relElevationRad = relElevationRad
		msg.piSimdis.getRCSdB.polarity = polarity
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.doubleValue)

	def loadDISCNFile(self, filename):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.loadDISCNFile = filename
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def setTextureLayerActive(self, layerId, active):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.terrain.setTextureLayerActive.layerId = layerId
		msg.piSimdis.terrain.setTextureLayerActive.active = active
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def setTextureLayerOpacity(self, layerId, scalar):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.terrain.setTextureLayerTransparency.layerId = layerId
		msg.piSimdis.terrain.setTextureLayerTransparency.scalar = scalar
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def setTextureLayerCloudTransparency(self, layerId, useCloudLayer, opaqueThreshold, clearThreshold):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.terrain.setTextureLayerCloudTransparency.layerId = layerId
		msg.piSimdis.terrain.setTextureLayerCloudTransparency.useCloudTransparency = useCloudLayer
		msg.piSimdis.terrain.setTextureLayerCloudTransparency.opaqueThreshold = opaqueThreshold
		msg.piSimdis.terrain.setTextureLayerCloudTransparency.clearThreshold = clearThreshold
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def setTextureLayerAltitude(self, layerId, altitude):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.terrain.setTextureLayerAltitude.layerId = layerId
		msg.piSimdis.terrain.setTextureLayerAltitude.altitude = altitude
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def addProjectedTextureRasterSet(self, projectorName):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.terrain.addSet.projectorName = projectorName
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.terrain.id)

	def addTextureLayer(self, sphereId, rasterSetId, layerName):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.terrain.addLayer.sphereId = sphereId
		msg.piSimdis.terrain.addLayer.rasterSetId = rasterSetId
		msg.piSimdis.terrain.addLayer.layerName = layerName
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.terrain.id)

	def removeTextureLayer(self, sphereId, layerId):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.terrain.removeLayer.sphereId = sphereId
		msg.piSimdis.terrain.removeLayer.layerId = layerId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def addAltitudeLayer(self, rasterSetId, layerName):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.terrain.addAltitudeLayer.rasterSetId = rasterSetId
		msg.piSimdis.terrain.addAltitudeLayer.layerName = layerName
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.terrain.id)

	def removeAltitudeLayer(self, layerId):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.terrain.removeAltitudeLayer.layerId = layerId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def setAltitudeLayerActive(self, layerId, active):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.terrain.setAltitudeLayerActive.layerId = layerId
		msg.piSimdis.terrain.setAltitudeLayerActive.active = active
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def addSqliteDbFileRasterSet(self, dbFileName):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.terrain.addSqliteDbFileRasterSet.dbFileName = dbFileName
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.terrain.id)

	def addGeoreferencedFileRasterSet(self, filename, srcAlpha, dstAlpha, srcSrs):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.terrain.addGeoreferencedFileRasterSet.filename = filename
		msg.piSimdis.terrain.addGeoreferencedFileRasterSet.srcAlpha = srcAlpha
		msg.piSimdis.terrain.addGeoreferencedFileRasterSet.dstAlpha = dstAlpha
		msg.piSimdis.terrain.addGeoreferencedFileRasterSet.srcSrs = srcSrs
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.terrain.id)

	def addInMemoryGeographicRasterSet(self, textureId, minLat, maxLat, minLon, maxLon, spansLon180):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.terrain.addInMemoryGeographicRasterSet.textureId = textureId
		msg.piSimdis.terrain.addInMemoryGeographicRasterSet.minLat = minLat
		msg.piSimdis.terrain.addInMemoryGeographicRasterSet.maxLat = maxLat
		msg.piSimdis.terrain.addInMemoryGeographicRasterSet.minLon = minLon
		msg.piSimdis.terrain.addInMemoryGeographicRasterSet.maxLon = maxLon
		msg.piSimdis.terrain.addInMemoryGeographicRasterSet.spansLon180 = spansLon180
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.terrain.id)

	def setRasterSetExtents(self, rasterSetId, minLat, maxLat, minLon, maxLon, spansLon180):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.terrain.setRasterSetExtents.rasterSetId = rasterSetId
		msg.piSimdis.terrain.setRasterSetExtents.minLat = minLat
		msg.piSimdis.terrain.setRasterSetExtents.maxLat = maxLat
		msg.piSimdis.terrain.setRasterSetExtents.minLon = minLon
		msg.piSimdis.terrain.setRasterSetExtents.maxLon = maxLon
		msg.piSimdis.terrain.setRasterSetExtents.spansLon180 = spansLon180
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def removeRasterSet(self, rasterSetId):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.terrain.removeRasterSet = rasterSetId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getLayerOption(self, layerId, optionName):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.terrain.getLayerOption.layerId = layerId
		msg.piSimdis.terrain.getLayerOption.settings.append(optionName)
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.terrain.layerOptions[0].valueDouble)

	def getLayerOptionBool(self, layerId, optionName):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.terrain.getLayerOption.layerId = layerId
		msg.piSimdis.terrain.getLayerOption.settings.append(optionName)
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.terrain.layerOptions[0].valueBool)

	def getLayerOptionString(self, layerId, optionName):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.terrain.getLayerOption.layerId = layerId
		msg.piSimdis.terrain.getLayerOption.settings.append(optionName)
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.terrain.layerOptions[0].valueString)

	def setLayerOption(self, layerId, optionName, value):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.terrain.setLayerOption.layerId = layerId
		opt = msg.piSimdis.terrain.setLayerOption.options.add()
		opt.opt = optionName
		opt.valueDouble = value
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def setLayerOptionBool(self, layerId, optionName, value):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.terrain.setLayerOption.layerId = layerId
		opt = msg.piSimdis.terrain.setLayerOption.options.add()
		opt.opt = optionName
		opt.valueBool = value
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def setLayerOptionString(self, layerId, optionName, value):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.terrain.setLayerOption.layerId = layerId
		opt = msg.piSimdis.terrain.setLayerOption.options.add()
		opt.opt = optionName
		opt.valueString = value
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getTerrainLayers(self):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.terrain.getTerrainLayers = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.terrain.idList)

	def getLayerName(self, layerId):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.terrain.getLayerName = layerId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.terrain.name)

	def getTextureLayerActive(self, layerId):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.terrain.getTextureLayerActive = layerId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.terrain.active)

	def getTextureLayerOpacity(self, layerId):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.terrain.getTextureLayerTransparency = layerId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.terrain.doubleVal)

	def getTextureLayerAltitude(self, layerId):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.terrain.getTextureLayerAltitude = layerId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.terrain.doubleVal)

	def getTextureLayerFilenameOrUrl(self, layerId):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.terrain.getTextureLayerFilenameOrUrl = layerId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.terrain.name)

	def geodeticToScreenXY(self, lat, lon, alt, view = "MainView"):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.geodeticToScreenXY.lat = lat
		msg.piSimdis.geodeticToScreenXY.lon = lon
		msg.piSimdis.geodeticToScreenXY.alt = alt
		msg.piSimdis.geodeticToScreenXY.view = view
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.screenXY)

	def getAltitudeForLL(self, lat, lon, vd):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.getAltitudeForLL.lat = lat
		msg.piSimdis.getAltitudeForLL.lon = lon
		msg.piSimdis.getAltitudeForLL.vd = vd
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.doubleValue)

	def setAntennaInfo(self, beamId, name, algo, polarity, filename, gain, power, freq):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.antenna.setAntennaInfo.beamId = beamId
		msg.piSimdis.antenna.setAntennaInfo.name = name
		msg.piSimdis.antenna.setAntennaInfo.algo = algo
		msg.piSimdis.antenna.setAntennaInfo.polarity = polarity
		msg.piSimdis.antenna.setAntennaInfo.filename = filename
		msg.piSimdis.antenna.setAntennaInfo.gain = gain
		msg.piSimdis.antenna.setAntennaInfo.power = power
		msg.piSimdis.antenna.setAntennaInfo.freq = freq
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def clearAntennaInfo(self, beamId):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.antenna.clearAntennaInfo = beamId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def setAntennaGain(self, beamId, gain):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.antenna.setAntennaGain.beamId = beamId
		msg.piSimdis.antenna.setAntennaGain.gain = gain
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def setAntennaPower(self, beamId, power):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.antenna.setAntennaPower.beamId = beamId
		msg.piSimdis.antenna.setAntennaPower.power = power
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def setAntennaFrequency(self, beamId, freq):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.antenna.setAntennaFrequency.beamId = beamId
		msg.piSimdis.antenna.setAntennaFrequency.freq = freq
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def setAntennaPolarity(self, beamId, polarity):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.antenna.setAntennaPolarity.beamId = beamId
		msg.piSimdis.antenna.setAntennaPolarity.polarity = polarity
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getAntennaName(self, beamId):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.antenna.getAntennaName = beamId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.antenna.antennaName)

	def getAntennaAlgorithm(self, beamId):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.antenna.getAntennaAlgorithm = beamId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.antenna.algorithm)

	def getAntennaFilename(self, beamId):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.antenna.getAntennaFilename = beamId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.antenna.filename)

	def getAntennaGain(self, beamId):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.antenna.getAntennaGain = beamId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.antenna.doubleValue)

	def getAntennaPatternGain(self, beamId, azRad, elRad):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.antenna.getAntennaPatternGain.beamId = beamId
		msg.piSimdis.antenna.getAntennaPatternGain.azRad = azRad
		msg.piSimdis.antenna.getAntennaPatternGain.elRad = elRad
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.antenna.doubleValue)

	def getAntennaPower(self, beamId):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.antenna.getAntennaPower = beamId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.antenna.doubleValue)

	def getAntennaFrequency(self, beamId):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.antenna.getAntennaFrequency = beamId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.antenna.doubleValue)

	def getAntennaPolarity(self, beamId):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.antenna.getAntennaPolarity = beamId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.antenna.polarity)

	def cycleEyeView(self, forwardCycle, viewport = ""):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.view.cycleEyeView.forwardCycle = forwardCycle
		msg.piSimdis.view.cycleEyeView.viewport = viewport
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def createView(self, viewString):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.view.createView = viewString
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def changeView(self, viewName, viewport = ""):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.view.changeView.viewName = viewName
		msg.piSimdis.view.changeView.viewport = viewport
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def setViewportFov(self, viewport, fovY, fovX=None):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.view.setFov.viewport = viewport
		msg.piSimdis.view.setFov.fovDeg = fovY
		if fovX is not None:
			msg.piSimdis.view.setFov.fovXDeg = fovX
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getCurrentViewport(self):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.view.getCurrentViewport = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.view.viewport)

	def getViewString(self, viewport = ""):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.view.getViewString = viewport
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.view.viewString)

	def deleteView(self, viewName):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.view.deleteView = viewName
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def loadViewFile(self, filename):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.view.loadViewFile = filename
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def saveViewFile(self, filename):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.view.saveViewFile = filename
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getOverheadMode(self, viewport = ""):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.view.getOverheadMode = viewport
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
				lambda r: r.piSimdis.view.overheadMode)

	def getCameraInfo(self, viewport = ""):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.view.getCameraInfo = viewport
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.view.cameraInfo)

	def getEyePosition(self, viewport = ""):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.view.getEyePosition = viewport
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.view.eyePosition)

	def viewMotion(self, newView, viewport, frames = None, duration = None):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.view.viewMotion.newView = newView
		msg.piSimdis.view.viewMotion.viewport = viewport
		if frames != None:
			msg.piSimdis.view.viewMotion.frames = frames
		if duration != None:
			msg.piSimdis.view.viewMotion.duration = duration
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def motionStop(self, viewport):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.view.motionStop = viewport
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def tetherCamera(self, viewport, entityId):
		msg = RemoteApi_pb2.Command()
		camInfoMsg = msg.piSimdis.view.setCameraInfo
		camInfoMsg.viewport = viewport
		camInfoMsg.tetherId = entityId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def setCameraRAE(self, viewport, range, az, el):
		msg = RemoteApi_pb2.Command()
		camInfoMsg = msg.piSimdis.view.setCameraInfo
		camInfoMsg.viewport = viewport
		camInfoMsg.cameraRAE.rangeMeters = range
		camInfoMsg.cameraRAE.azimuthRad = az
		camInfoMsg.cameraRAE.elevationRad = el
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def savePreferencesStream(self):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.savePreferencesStream = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.preferencesBytes)

	def loadPreferencesStream(self, prefsData):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.loadPreferencesStream = prefsData
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def saveEntityPreferencesStream(self, id):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.saveEntityPreferencesStream = id
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.preferencesBytes)

	def loadEntityPreferencesStream(self, id, prefsData):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.loadEntityPreferencesStream.id = id
		msg.piSimdis.loadEntityPreferencesStream.prefsStream = prefsData
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def addBookmarkEvent(self, time, category, text, id = None):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.bookmark.addBookmarkEvent.time = time
		msg.piSimdis.bookmark.addBookmarkEvent.category = category
		msg.piSimdis.bookmark.addBookmarkEvent.text = text
		if id != None:
			msg.piSimdis.bookmark.addBookmarkEvent.suppliedId = id
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
		lambda r: r.piSimdis.bookmark.id)

	def removeBookmarkEvent(self, id):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.bookmark.removeBookmarkEvent = id
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg), ReturnCodeOnly)

	def getAllBookmarkEvents(self):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.bookmark.getAllBookmarkEvents = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.bookmark.idList)

	def getBookmarkEventData(self, id):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.bookmark.getBookmarkEventData = id
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.bookmark.bookmarkData)

	def getBookmarkEvent(self, id):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.bookmark.getBookmarkEvent = id
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.bookmark.bookmarkData)

	def getAllGogObjects(self, storageType):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.draw.getAllGogObjects = storageType
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.draw.idList)

	def loadGogStream(self, gogData, platformId, suppliedId):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.draw.loadGogStream.gogData = gogData
		msg.piSimdis.draw.loadGogStream.platformId = platformId
		msg.piSimdis.draw.loadGogStream.suppliedId = suppliedId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.draw.id)

	def getGogHost(self, gogData):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.draw.getGogHost = gogData
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.draw.id)

	def serializeGogToStream(self, id):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.draw.serializeGogToStream = id
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.draw.data)

	def serializeToStream(self, format, options=[]):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.file.serializeToStream.format = format
		for option in options:
			addedOption = msg.piSimdis.file.serializeToStream.options.add()
			addedOption.key = option.key
			addedOption.value = option.value
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.file.data)

	def deserializeStream(self, format, data, options=[]):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.file.deserializeStream.format = format
		msg.piSimdis.file.deserializeStream.data = data
		for option in options:
			addedOption = msg.piSimdis.file.deserializeStream.options.add()
			addedOption.key = option.key
			addedOption.value = option.value
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getFileMetadata(self, format):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.file.getFileMetadata = format
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.file.metadata)

	def loadMediaFile(self, filename):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.file.loadMediaFile.filename = filename
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def addTextureStream(self, fileData, border, name, id = None):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.texture.addTextureStream.fileData = fileData
		msg.piSimdis.texture.addTextureStream.border = border
		msg.piSimdis.texture.addTextureStream.name = name
		if id != None:
			msg.piSimdis.texture.addTextureStream.suppliedId = id
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.texture.id)

	def updateTextureStream(self, textureId, fileData, xoffset, yoffset):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.texture.updateTextureStream.textureId = textureId
		msg.piSimdis.texture.updateTextureStream.fileData = fileData
		msg.piSimdis.texture.updateTextureStream.xoffset = xoffset
		msg.piSimdis.texture.updateTextureStream.yoffset = yoffset
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def updateTexture(self, textureId, target, level, xoffset, yoffset, width, height, format, type, pixels):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.texture.updateTexture.textureId = textureId
		msg.piSimdis.texture.updateTexture.target = target
		msg.piSimdis.texture.updateTexture.level = level
		msg.piSimdis.texture.updateTexture.xoffset = xoffset
		msg.piSimdis.texture.updateTexture.yoffset = yoffset
		msg.piSimdis.texture.updateTexture.width = width
		msg.piSimdis.texture.updateTexture.height = height
		msg.piSimdis.texture.updateTexture.format = format
		msg.piSimdis.texture.updateTexture.type = type
		msg.piSimdis.texture.updateTexture.pixels = pixels
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def removeTexture(self, id):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.texture.removeTexture.id = id
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def setPlatformTextureIcon(self, platformId, textureId):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.texture.setPlatformTextureIcon.platformId = platformId
		msg.piSimdis.texture.setPlatformTextureIcon.textureId = textureId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def setEntityAcceptsProjector(self, entityId, projectorId, drawOnTop):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.draw.setEntityAcceptsProjector.entityId = entityId
		msg.piSimdis.draw.setEntityAcceptsProjector.projectorId = projectorId
		msg.piSimdis.draw.setEntityAcceptsProjector.drawOnTop = drawOnTop
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getRCSData(self, platformId):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.getRCSData = platformId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.rcsData)

	def setRCSData(self, platformId, polarity = None, freqMhz = None):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.setRCSData.id = platformId
		if (polarity is not None):
			msg.piSimdis.setRCSData.rcsData.polarity = polarity
		if (freqMhz is not None):
			msg.piSimdis.setRCSData.rcsData.freqMhz = freqMhz
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getEntityHooks(self):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.getEntityHooks = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.entityIds)

	def setEntityHooked(self, platformId, hooked):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.setEntityHooked.id = platformId
		msg.piSimdis.setEntityHooked.hooked = hooked
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def isEntityHooked(self, platformId):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.isEntityHooked = platformId
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.state)

	def unhookAllEntities(self):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.unhookAllEntities = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def createSim3dLandmark(self, pos, ori, iconName, alphaVolume, scaleVolume, beginTime, endTime, labelString):
		msg = RemoteApi_pb2.Command()
		if pos is not None:
			msg.piSimdis.landmark.createSim3dLandmark.position.x = pos[0]
			msg.piSimdis.landmark.createSim3dLandmark.position.y = pos[1]
			msg.piSimdis.landmark.createSim3dLandmark.position.z = pos[2]
		if ori is not None:
			msg.piSimdis.landmark.createSim3dLandmark.orientation.x = ori[0]
			msg.piSimdis.landmark.createSim3dLandmark.orientation.y = ori[1]
			msg.piSimdis.landmark.createSim3dLandmark.orientation.z = ori[2]
		msg.piSimdis.landmark.createSim3dLandmark.iconName = iconName
		msg.piSimdis.landmark.createSim3dLandmark.alphaVolume = alphaVolume
		if scaleVolume is not None:
			msg.piSimdis.landmark.createSim3dLandmark.scaleVolume.x = scaleVolume[0]
			msg.piSimdis.landmark.createSim3dLandmark.scaleVolume.y = scaleVolume[1]
			msg.piSimdis.landmark.createSim3dLandmark.scaleVolume.z = scaleVolume[2]
		msg.piSimdis.landmark.createSim3dLandmark.beginTime = beginTime
		msg.piSimdis.landmark.createSim3dLandmark.endTime = endTime
		msg.piSimdis.landmark.createSim3dLandmark.labelString = labelString
		msg.piSimdis.landmark.createSim3dLandmark.referenceFrame.system = 3 # lla
		# Note that textSettings is left at default values
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.landmark.id)

	def removeSim3dLandmark(self, whichMarker):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.landmark.removeSim3dLandmark = whichMarker
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def updateSim3dLandmark(self, whichMarker, pos, ori, iconName, alphaVolume, scaleVolume, beginTime, endTime, labelString):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.landmark.updateSim3dLandmark.whichMarker = whichMarker
		if pos is not None:
			msg.piSimdis.landmark.updateSim3dLandmark.landmark.position.x = pos[0]
			msg.piSimdis.landmark.updateSim3dLandmark.landmark.position.y = pos[1]
			msg.piSimdis.landmark.updateSim3dLandmark.landmark.position.z = pos[2]
		if ori is not None:
			msg.piSimdis.landmark.updateSim3dLandmark.landmark.orientation.x = ori[0]
			msg.piSimdis.landmark.updateSim3dLandmark.landmark.orientation.y = ori[1]
			msg.piSimdis.landmark.updateSim3dLandmark.landmark.orientation.z = ori[2]
		msg.piSimdis.landmark.updateSim3dLandmark.landmark.iconName = iconName
		msg.piSimdis.landmark.updateSim3dLandmark.landmark.alphaVolume = alphaVolume
		if scaleVolume is not None:
			msg.piSimdis.landmark.updateSim3dLandmark.landmark.scaleVolume.x = scaleVolume[0]
			msg.piSimdis.landmark.updateSim3dLandmark.landmark.scaleVolume.y = scaleVolume[1]
			msg.piSimdis.landmark.updateSim3dLandmark.landmark.scaleVolume.z = scaleVolume[2]
		msg.piSimdis.landmark.updateSim3dLandmark.landmark.beginTime = beginTime
		msg.piSimdis.landmark.updateSim3dLandmark.landmark.endTime = endTime
		msg.piSimdis.landmark.updateSim3dLandmark.landmark.labelString = labelString
		msg.piSimdis.landmark.updateSim3dLandmark.landmark.referenceFrame.system = 3 # lla
		# Note that textSettings is left at default values
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getSim3dLandmark(self, whichMarker):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.landmark.getSim3dLandmark = whichMarker
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.landmark.landmark)

	def setSim3dLandmarkColor(self, whichMarker, useOverride, color):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.landmark.setSim3dLandmarkColor.whichMarker = whichMarker
		msg.piSimdis.landmark.setSim3dLandmarkColor.override.useOverride = useOverride
		msg.piSimdis.landmark.setSim3dLandmarkColor.override.color = color
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getSim3dLandmarkOverrideColor(self, whichMarker):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.landmark.getSim3dLandmarkOverrideColor = whichMarker
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.landmark.overrideColor)

	def setLandmarkPreference(self, whichMarker, fieldPath, value):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.landmark.setLandmarkPreference.whichMarker = whichMarker
		pair = msg.piSimdis.landmark.setLandmarkPreference.fieldValPairs.add()
		pair.fieldPath = fieldPath
		pair.value = value
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getLandmarkPreference(self, whichMarker, fieldPath):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.landmark.getLandmarkPreference.whichMarker = whichMarker
		msg.piSimdis.landmark.getLandmarkPreference.fieldPaths.append(fieldPath)
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.landmark.values[0])

	def registerHudWindow(self, name, defaultPositionPctX, defaultPositionPctY):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.hudWindow.registerHudWindow.name = name
		msg.piSimdis.hudWindow.registerHudWindow.position.xPercent = defaultPositionPctX
		msg.piSimdis.hudWindow.registerHudWindow.position.yPercent = defaultPositionPctY
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def removeHudWindow(self, name):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.hudWindow.removeHudWindow = name
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getAllHudWindows(self):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.hudWindow.getAllHudWindows = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.hudWindow.names)

	def getHudWindowPosition(self, name):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.hudWindow.getHudWindowPosition = name
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.hudWindow.position)

	def setHudWindowPosition(self, name, positionPctX, positionPctY):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.hudWindow.setHudWindowPosition.name = name
		msg.piSimdis.hudWindow.setHudWindowPosition.position.xPercent = positionPctX
		msg.piSimdis.hudWindow.setHudWindowPosition.position.yPercent = positionPctY
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def resetHudWindowPosition(self, name):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.hudWindow.resetHudWindowPosition = name
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def resetAllHudWindowPositions(self):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.hudWindow.resetAllHudWindowPositions = True
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getHudWindowSize(self, name):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.hudWindow.getHudWindowSize = name
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.hudWindow.size)

	def setHudWindowSize(self, name, leftPx, bottomPx, rightPx, topPx):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.hudWindow.setHudWindowSize.name = name
		msg.piSimdis.hudWindow.setHudWindowSize.size.leftPixel = leftPx
		msg.piSimdis.hudWindow.setHudWindowSize.size.bottomPixel = bottomPx
		msg.piSimdis.hudWindow.setHudWindowSize.size.rightPixel = rightPx
		msg.piSimdis.hudWindow.setHudWindowSize.size.topPixel = topPx
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def enableReplay(self, enable, source, reason):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.enableReplay.enable = enable
		msg.piSimdis.enableReplay.source = source
		msg.piSimdis.enableReplay.reason = reason
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getMagneticVariance(self, latInRadians, lonInRadians, altInMeters, time):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.getMagneticVariance.latInRadians = latInRadians
		msg.piSimdis.getMagneticVariance.lonInRadians = lonInRadians
		msg.piSimdis.getMagneticVariance.altInMeters = altInMeters
		msg.piSimdis.getMagneticVariance.time = time
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.doubleValue)

	def getMagneticVarianceById(self, id, time):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.getMagneticVariance.id = id
		msg.piSimdis.getMagneticVariance.time = time
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.doubleValue)

	def attachShader(self, context, platformId, shaderName, shader):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.shader.attachShader.context = context
		msg.piSimdis.shader.attachShader.id = platformId
		msg.piSimdis.shader.attachShader.shaderName = shaderName
		msg.piSimdis.shader.attachShader.shader = shader
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def setShaderValueB(self, context, platformId, name, value):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.shader.setShaderValueB.context = context
		msg.piSimdis.shader.setShaderValueB.id = platformId
		msg.piSimdis.shader.setShaderValueB.name = name
		msg.piSimdis.shader.setShaderValueB.values.append(value)
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def setShaderValueI(self, context, platformId, name, value):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.shader.setShaderValueI.context = context
		msg.piSimdis.shader.setShaderValueI.id = platformId
		msg.piSimdis.shader.setShaderValueI.name = name
		msg.piSimdis.shader.setShaderValueI.values.append(value)
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def setShaderValueF(self, context, platformId, name, value):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.shader.setShaderValueF.context = context
		msg.piSimdis.shader.setShaderValueF.id = platformId
		msg.piSimdis.shader.setShaderValueF.name = name
		msg.piSimdis.shader.setShaderValueF.values.append(value)
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			ReturnCodeOnly)

	def getShaderValueB(self, context, platformId, name):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.shader.getShaderValueB.context = context
		msg.piSimdis.shader.getShaderValueB.id = platformId
		msg.piSimdis.shader.getShaderValueB.name = name
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.shader.booleans)

	def getShaderValueI(self, context, platformId, name):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.shader.getShaderValueI.context = context
		msg.piSimdis.shader.getShaderValueI.id = platformId
		msg.piSimdis.shader.getShaderValueI.name = name
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.shader.integers)

	def getShaderValueF(self, context, platformId, name):
		msg = RemoteApi_pb2.Command()
		msg.piSimdis.shader.getShaderValueF.context = context
		msg.piSimdis.shader.getShaderValueF.id = platformId
		msg.piSimdis.shader.getShaderValueF.name = name
		return PluginApi.ResponseHandler(self.client_, self.client_.send(msg),
			lambda r: r.piSimdis.shader.floats)
